# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/python/saved_model/saved_object_graph.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.protobuf import checkpointable_object_graph_pb2 as tensorflow_dot_core_dot_protobuf_dot_checkpointable__object__graph__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/python/saved_model/saved_object_graph.proto',
  package='tensorflow',
  syntax='proto3',
  serialized_options=_b('\370\001\001'),
  serialized_pb=_b('\n6tensorflow/python/saved_model/saved_object_graph.proto\x12\ntensorflow\x1a:tensorflow/core/protobuf/checkpointable_object_graph.proto\":\n\x10SavedObjectGraph\x12&\n\x05nodes\x18\x01 \x03(\x0b\x32\x17.tensorflow.SavedObject\"\x86\x03\n\x0bSavedObject\x12\\\n\x08\x63hildren\x18\x01 \x03(\x0b\x32J.tensorflow.CheckpointableObjectGraph.CheckpointableObject.ObjectReference\x12h\n\x0eslot_variables\x18\x03 \x03(\x0b\x32P.tensorflow.CheckpointableObjectGraph.CheckpointableObject.SlotVariableReference\x12\x32\n\x0buser_object\x18\x04 \x01(\x0b\x32\x1b.tensorflow.SavedUserObjectH\x00\x12\'\n\x05\x61sset\x18\x05 \x01(\x0b\x32\x16.tensorflow.SavedAssetH\x00\x12\x38\n\x08\x66unction\x18\x06 \x01(\x0b\x32$.tensorflow.SavedPolymorphicFunctionH\x00\x42\x06\n\x04kindJ\x04\x08\x02\x10\x03R\nattributes\"\x11\n\x0fSavedUserObject\"*\n\nSavedAsset\x12\x1c\n\x14\x61sset_file_def_index\x18\x01 \x01(\r\"^\n\x18SavedPolymorphicFunction\x12\x42\n\x14monomorphic_function\x18\x01 \x03(\x0b\x32$.tensorflow.SavedMonomorphicFunction\"5\n\x18SavedMonomorphicFunction\x12\x19\n\x11\x63oncrete_function\x18\x01 \x01(\tB\x03\xf8\x01\x01\x62\x06proto3')
  ,
  dependencies=[tensorflow_dot_core_dot_protobuf_dot_checkpointable__object__graph__pb2.DESCRIPTOR,])




_SAVEDOBJECTGRAPH = _descriptor.Descriptor(
  name='SavedObjectGraph',
  full_name='tensorflow.SavedObjectGraph',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nodes', full_name='tensorflow.SavedObjectGraph.nodes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=130,
  serialized_end=188,
)


_SAVEDOBJECT = _descriptor.Descriptor(
  name='SavedObject',
  full_name='tensorflow.SavedObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='children', full_name='tensorflow.SavedObject.children', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='slot_variables', full_name='tensorflow.SavedObject.slot_variables', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_object', full_name='tensorflow.SavedObject.user_object', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asset', full_name='tensorflow.SavedObject.asset', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='function', full_name='tensorflow.SavedObject.function', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='kind', full_name='tensorflow.SavedObject.kind',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=191,
  serialized_end=581,
)


_SAVEDUSEROBJECT = _descriptor.Descriptor(
  name='SavedUserObject',
  full_name='tensorflow.SavedUserObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=583,
  serialized_end=600,
)


_SAVEDASSET = _descriptor.Descriptor(
  name='SavedAsset',
  full_name='tensorflow.SavedAsset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='asset_file_def_index', full_name='tensorflow.SavedAsset.asset_file_def_index', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=602,
  serialized_end=644,
)


_SAVEDPOLYMORPHICFUNCTION = _descriptor.Descriptor(
  name='SavedPolymorphicFunction',
  full_name='tensorflow.SavedPolymorphicFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='monomorphic_function', full_name='tensorflow.SavedPolymorphicFunction.monomorphic_function', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=646,
  serialized_end=740,
)


_SAVEDMONOMORPHICFUNCTION = _descriptor.Descriptor(
  name='SavedMonomorphicFunction',
  full_name='tensorflow.SavedMonomorphicFunction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='concrete_function', full_name='tensorflow.SavedMonomorphicFunction.concrete_function', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=742,
  serialized_end=795,
)

_SAVEDOBJECTGRAPH.fields_by_name['nodes'].message_type = _SAVEDOBJECT
_SAVEDOBJECT.fields_by_name['children'].message_type = tensorflow_dot_core_dot_protobuf_dot_checkpointable__object__graph__pb2._CHECKPOINTABLEOBJECTGRAPH_CHECKPOINTABLEOBJECT_OBJECTREFERENCE
_SAVEDOBJECT.fields_by_name['slot_variables'].message_type = tensorflow_dot_core_dot_protobuf_dot_checkpointable__object__graph__pb2._CHECKPOINTABLEOBJECTGRAPH_CHECKPOINTABLEOBJECT_SLOTVARIABLEREFERENCE
_SAVEDOBJECT.fields_by_name['user_object'].message_type = _SAVEDUSEROBJECT
_SAVEDOBJECT.fields_by_name['asset'].message_type = _SAVEDASSET
_SAVEDOBJECT.fields_by_name['function'].message_type = _SAVEDPOLYMORPHICFUNCTION
_SAVEDOBJECT.oneofs_by_name['kind'].fields.append(
  _SAVEDOBJECT.fields_by_name['user_object'])
_SAVEDOBJECT.fields_by_name['user_object'].containing_oneof = _SAVEDOBJECT.oneofs_by_name['kind']
_SAVEDOBJECT.oneofs_by_name['kind'].fields.append(
  _SAVEDOBJECT.fields_by_name['asset'])
_SAVEDOBJECT.fields_by_name['asset'].containing_oneof = _SAVEDOBJECT.oneofs_by_name['kind']
_SAVEDOBJECT.oneofs_by_name['kind'].fields.append(
  _SAVEDOBJECT.fields_by_name['function'])
_SAVEDOBJECT.fields_by_name['function'].containing_oneof = _SAVEDOBJECT.oneofs_by_name['kind']
_SAVEDPOLYMORPHICFUNCTION.fields_by_name['monomorphic_function'].message_type = _SAVEDMONOMORPHICFUNCTION
DESCRIPTOR.message_types_by_name['SavedObjectGraph'] = _SAVEDOBJECTGRAPH
DESCRIPTOR.message_types_by_name['SavedObject'] = _SAVEDOBJECT
DESCRIPTOR.message_types_by_name['SavedUserObject'] = _SAVEDUSEROBJECT
DESCRIPTOR.message_types_by_name['SavedAsset'] = _SAVEDASSET
DESCRIPTOR.message_types_by_name['SavedPolymorphicFunction'] = _SAVEDPOLYMORPHICFUNCTION
DESCRIPTOR.message_types_by_name['SavedMonomorphicFunction'] = _SAVEDMONOMORPHICFUNCTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SavedObjectGraph = _reflection.GeneratedProtocolMessageType('SavedObjectGraph', (_message.Message,), dict(
  DESCRIPTOR = _SAVEDOBJECTGRAPH,
  __module__ = 'tensorflow.python.saved_model.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.SavedObjectGraph)
  ))
_sym_db.RegisterMessage(SavedObjectGraph)

SavedObject = _reflection.GeneratedProtocolMessageType('SavedObject', (_message.Message,), dict(
  DESCRIPTOR = _SAVEDOBJECT,
  __module__ = 'tensorflow.python.saved_model.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.SavedObject)
  ))
_sym_db.RegisterMessage(SavedObject)

SavedUserObject = _reflection.GeneratedProtocolMessageType('SavedUserObject', (_message.Message,), dict(
  DESCRIPTOR = _SAVEDUSEROBJECT,
  __module__ = 'tensorflow.python.saved_model.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.SavedUserObject)
  ))
_sym_db.RegisterMessage(SavedUserObject)

SavedAsset = _reflection.GeneratedProtocolMessageType('SavedAsset', (_message.Message,), dict(
  DESCRIPTOR = _SAVEDASSET,
  __module__ = 'tensorflow.python.saved_model.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.SavedAsset)
  ))
_sym_db.RegisterMessage(SavedAsset)

SavedPolymorphicFunction = _reflection.GeneratedProtocolMessageType('SavedPolymorphicFunction', (_message.Message,), dict(
  DESCRIPTOR = _SAVEDPOLYMORPHICFUNCTION,
  __module__ = 'tensorflow.python.saved_model.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.SavedPolymorphicFunction)
  ))
_sym_db.RegisterMessage(SavedPolymorphicFunction)

SavedMonomorphicFunction = _reflection.GeneratedProtocolMessageType('SavedMonomorphicFunction', (_message.Message,), dict(
  DESCRIPTOR = _SAVEDMONOMORPHICFUNCTION,
  __module__ = 'tensorflow.python.saved_model.saved_object_graph_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.SavedMonomorphicFunction)
  ))
_sym_db.RegisterMessage(SavedMonomorphicFunction)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
