# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Inception-ResNet V2 model for Keras.

"""

from __future__ import print_function as _print_function

from tensorflow.python.keras.applications import InceptionResNetV2
from tensorflow.python.keras.applications.inception_resnet_v2 import decode_predictions
from tensorflow.python.keras.applications.inception_resnet_v2 import preprocess_input

del _print_function
